(*@PROPERTIES_EX@
TYPE: POU
LOCALE: 0
IEC_LANGUAGE: ST
PLC_TYPE: independent
PROC_TYPE: independent
GROUP: HLK
*)
(*@KEY@:DESCRIPTION*)
version 1.2	13. mar. 2009
programmer 	hugo
tested by	tobias

water_cp calculates the specific heat capacity of liquid water from 0..100 C
(*@KEY@:END_DESCRIPTION*)
FUNCTION_BLOCK WATER_CP

(*Group:Default*)


VAR_INPUT
	T :	REAL;
END_VAR


VAR_OUTPUT
	CAP :	REAL;
END_VAR


VAR
	Pts :	INT;
	data :	oscat_XY;
	FB_LINEAR_INT :	LINEAR_INT;
END_VAR


(*@KEY@: WORKSHEET
NAME: WATER_CP
IEC_LANGUAGE: ST
*)
IF Pts = 0 THEN
    Pts := 10;

    data[01][0] := REAL#0.0;
    data[01][1] := REAL#4.228;
    data[02][0] := REAL#05.0;
    data[02][1] := REAL#4.200;
    data[03][0] := REAL#10.0;
    data[03][1] := REAL#4.188;
    data[04][0] := REAL#15.0;
    data[04][1] := REAL#4.184;
    data[05][0] := REAL#50.0;
    data[05][1] := REAL#4.181;
    data[06][0] := REAL#60.0;
    data[06][1] := REAL#4.183;
    data[07][0] := REAL#70.0;
    data[07][1] := REAL#4.187;
    data[08][0] := REAL#80.0;
    data[08][1] := REAL#4.194;
    data[09][0] := REAL#90.0;
    data[09][1] := REAL#4.204;
    data[10][0] := REAL#100.0;
    data[10][1] := REAL#4.220;

END_IF;

FB_LINEAR_INT(X:=T,XY:=Data,Pts:=Pts);
cap:=FB_LINEAR_INT.LINEAR_INT;



(*
    C - die Celsius-Temperatur in Grad Celsius
    v  das spezifische Volumen in Kubikdezimeter je Kilogramm dm/kg
    h  die spezifische Enthalpie in Kilojoule je Kilogramm 	kJ/kg
    u  die spezifische Innere Energie in Kilojoule je Kilogramm 	kJ/kg
    s  die spezifische Entropie in Kilojoule je Kilogramm mal Kelvin 	kJ/(kgK)
    cp - die spezifische Wrmekapazitt bei konstantem Druck in Kilojoule je Kilogramm mal Kelvin 	kJ/(kgK)
    Y  Volumenausdehnungskoeffizient in 10-3 durch Kelvin  10-3/K
    L  Wrmeleitfhigkeit in Milliwatt je Meter mal Kelvin  mW / (mK) 
    n  Viskositt in Mikropascal mal Sekunde uPas 	
    s ?  Oberflchenspannung in Millinewton je Meter  mN/m


C 		v 				h 			u 		s 			cp 		Y 		L		n		s	
-20		1,006580	
0 		1,000160 		0,06 		-0,04 	-0,0001 	4,228 	-0,080 	561,0 	1792 	75,65
4		1,000028
5 		1,0000 		21,1 		21,0 	0,076 		4,200 	0,011 	570,6 	1518 	74,95
10 		1,0003 		42,1 		42,0 	0,151 		4,188 	0,087 	580,0 	1306 	74,22
15 		1,0009 		63,0 		62,9 	0,224 		4,184 	0,152 	589,4 	1137 	73,49
20 		1,001797 		83,9 		83,8 	0,296 		4,183 	0,209 	598,4 	1001 	72,74
25 		1,0029 		104,8 		104,7 	0,367 		4,183 	0,259 	607,2 	890,4 	71,98
30 		1,0044 		125,8 		125,7 	0,437 		4,183 	0,305 	615,5 	797,7 	71,20
35 		1,0060 		146,7 		146,6 	0,505 		4,183 	0,347 	623,3 	719,6 	70,41
40 		1,007842 		167,58 	167,5 	0,572 		4,182 	0,386 	630,6 	653,3 	69,60
45 		1,0099 		188,5 		188,4 	0,638 		4,182 	0,423 	637,3 	596,3 	68,78
50 		1,0121 		209,4 		209,3 	0,704 		4,181 	0,457 	643,6 	547,1 	67,95
60 		1,017089 		251,2 		251,1 	0,831 		4,183 	0,522 	654,4 	466,6 	66,24
70 		1,0227 		293,1 		293,0 	0,955 		4,187 	0,583 	663,1 	404,1 	64,49
80 		1,029027 		335,0 		334,9 	1,075 		4,194 	0,640 	670,0 	354,5 	62,68
90 		1,0359 		377,0 		376,9 	1,193 		4,204 	0,696 	675,3 	314,6 	60,82
99,63 	1,0431 		417,5 		417,4 	1,303 		4,217 	0,748 	679,0 	283,0 	58,99
100	1,043453		419,1

Stoffdaten von Wasser Quelle Wickipedia

Wasser hat unter Normaldruck seine grte Dichte von 1000 Kilogramm pro Kubikmeter bei 3,98 C und zeigt damit eine Dichteanomalie. 
Diese besteht darin, dass sich Wasser unterhalb von 3,98 C bei weiterer Temperaturverringerung, auch beim Wechsel zum festen 
Aggregatzustand, wieder ausdehnt, was man nur von wenigen Stoffen kennt. Die derzeit genauesten publizierten Werte fr die maximale 
Dichte liegen bei (999,974950  0,00084) kg/m und bei (3,983  0,00067) C fr die Temperatur, bei der dieser Wert erreicht wird. 
Die Werte stellen einen Mittelwert der von verschiedenen physikalischen Instituten verffentlichten Zahlen dar (Stand 2001).

Die Berechnung der Dichte von luftfreiem Wasser DLF in Abhngigkeit von der Temperatur T ([T] = C) kann mit Hilfe der folgenden 
Virialgleichung erfolgen:
: \frac{D_{\rm LF}}{\rm g/l} = \frac{a_0 + a_1 T + a_2 T^2 + a_3 T^3 + a_4 T^4 + a_5 T^5}{1 + b T}.
mit den Koeffizienten: a0 = 999,83952; a1 = 16,952577 (C)-1; a2 = -7,990512710-3 (C)-2; a3 = -4,624175710-5 (C)-3; 
a4 = 1,058460110-7 (C)-4; a5 = -2,810300610-10 (C)-5 und b = 0,0168872. 
Fr die Berechnung der Dichte von luftgesttigtem Wasser korrigiert man den Wert nach 
DLG/(g/l) = DLF/(g/l) - 0,004612 + 0,000106 (C)-1T. (Quelle: PTB-Mitteilungen 100/3-90)

*)

(* revision history

hm	29.10.2007	rev 1.0
	original version

hm	27. dec 2007	rev 1.1
	increased accuracy

hm	13. mar. 2009	rev 1.2
	real constants updated to new systax using dot

*)
(*@KEY@: END_WORKSHEET *)
END_FUNCTION_BLOCK
